@@ -8,6 +8,7 @@ import os
 import re
 import sys
 import textwrap
+import token
 import unittest
 from time import time
 
@@ -28,7 +29,16 @@ class TestMarkTokens(unittest.TestCase):
   module = ast
 
   def create_mark_checker(self, source, verify=True):
-    checker = tools.MarkChecker(self.create_asttokens(source))
+    atok = self.create_asttokens(source)
+    checker = tools.MarkChecker(atok)
+
+    # The last token should always be an ENDMARKER
+    # None of the nodes should contain that token
+    assert atok.tokens[-1].type == token.ENDMARKER
+    if atok.text:  # except for empty files
+      for node in checker.all_nodes:
+        assert node.last_token.type != token.ENDMARKER
+
     if verify:
       checker.verify_all_nodes(self)
     return checker
@@ -123,7 +133,10 @@ b +     # line3
     m = self.create_mark_checker(source, verify=False)
     tested_nodes = m.verify_all_nodes(self)
 
-    exp_index = (0 if six.PY2 else 1) + (2 if self.is_astroid_test else 0)
+    exp_index = (0 if six.PY2 else 1) + (3 if self.is_astroid_test else 0)
+    # For ast on Python 3.9, slices are expressions, we handle them and test them.
+    if not self.is_astroid_test and issubclass(ast.Slice, ast.expr):
+      exp_index += 1
     exp_tested_nodes = self.expect_tested_nodes[path][exp_index]
     self.assertEqual(tested_nodes, exp_tested_nodes)
 
@@ -131,21 +144,21 @@ b +     # line3
   # There is not too much need to verify these counts. The main reason is: if we find that some
   # change reduces the count by a lot, it's a red flag that the test is now covering fewer nodes.
   expect_tested_nodes = {
-    #                                   AST       | Astroid
-    #                                   Py2   Py3 | Py2   Py3
-    'astroid/__init__.py':            ( 4,    4,    4,    4,   ),
-    'astroid/absimport.py':           ( 4,    3,    4,    3,   ),
-    'astroid/all.py':                 ( 21,   23,   21,   23,  ),
-    'astroid/clientmodule_test.py':   ( 75,   67,   69,   69,  ),
-    'astroid/descriptor_crash.py':    ( 30,   28,   30,   30,  ),
-    'astroid/email.py':               ( 3,    3,    1,    1,   ),
-    'astroid/format.py':              ( 64,   61,   62,   62,  ),
-    'astroid/module.py':              ( 185,  174,  171,  171, ),
-    'astroid/module2.py':             ( 248,  253,  240,  253, ),
-    'astroid/noendingnewline.py':     ( 57,   59,   57,   63,  ),
-    'astroid/notall.py':              ( 15,   17,   15,   17,  ),
-    'astroid/recursion.py':           ( 6,    6,    4,    4,   ),
-    'astroid/suppliermodule_test.py': ( 20,   17,   18,   18,  ),
+    #                                   AST                  | Astroid
+    #                                   Py2   Py3  Py3+slice | Py2   Py3
+    'astroid/__init__.py':            ( 4,    4,   4,          4,    4,   ),
+    'astroid/absimport.py':           ( 4,    3,   3,          4,    3,   ),
+    'astroid/all.py':                 ( 21,   23,  23,         21,   23,  ),
+    'astroid/clientmodule_test.py':   ( 75,   67,  67,         69,   69,  ),
+    'astroid/descriptor_crash.py':    ( 30,   28,  28,         30,   30,  ),
+    'astroid/email.py':               ( 3,    3,   3,          1,    1,   ),
+    'astroid/format.py':              ( 64,   61,  61,         62,   62,  ),
+    'astroid/module.py':              ( 185,  174, 174,        171,  171, ),
+    'astroid/module2.py':             ( 248,  253, 255,        240,  253, ),
+    'astroid/noendingnewline.py':     ( 57,   59,  59,         57,   63,  ),
+    'astroid/notall.py':              ( 15,   17,  17,         15,   17,  ),
+    'astroid/recursion.py':           ( 6,    6,   6,          4,    4,   ),
+    'astroid/suppliermodule_test.py': ( 20,   17,  17,         18,   18,  ),
   }
 
   # This set of methods runs verifications for the variety of syntax constructs used in the
@@ -271,13 +284,14 @@ bar = ('x y z'   # comment2
       # Test of https://bitbucket.org/plas/thonny/issues/162/weird-range-marker-crash-with-non-ascii
       # Only on PY3 because Py2 doesn't support unicode identifiers.
       for source in (
+        "℘·2=1+a℘·b+a℘·2b",  # example from https://github.com/python/cpython/issues/68382
         "sünnikuupäev=str((18+int(isikukood[0:1])-1)//2)+isikukood[1:3]",
         "sünnikuupaev=str((18+int(isikukood[0:1])-1)//2)+isikukood[1:3]"):
         m = self.create_mark_checker(source)
         self.assertEqual(m.view_nodes_at(1, 0), {
           "Module:%s" % source,
           "Assign:%s" % source,
-          "%s:%s" % ("AssignName" if self.is_astroid_test else "Name", source[:12])
+          "%s:%s" % ("AssignName" if self.is_astroid_test else "Name", source.split("=")[0]),
         })
 
 
@@ -532,6 +546,20 @@ elif 2: b
     """
     self.create_mark_checker(source)
 
+
+  def test_statements_with_semicolons(self):
+    source = """
+a; b; c(
+  17
+); d # comment1; comment2
+if 2: a; b; # comment3
+    """
+    m = self.create_mark_checker(source)
+    self.assertEqual(
+      [m.atok.get_text(n) for n in m.all_nodes if util.is_stmt(n)],
+      ['a', 'b', 'c(\n  17\n)', 'd', 'if 2: a; b', 'a', 'b'])
+
+
   def test_complex_numbers(self):
     source = """
 1
@@ -607,8 +635,19 @@ j  # not a complex number, just a name
         except OSError:
           continue
 
-        # Astroid fails with a syntax error if a type comment is on its own line
-        if self.is_astroid_test and re.search(r'^\s*# type: ', source, re.MULTILINE):
+        if self.is_astroid_test and (
+            # Astroid fails with a syntax error if a type comment is on its own line
+            re.search(r'^\s*# type: ', source, re.MULTILINE)
+            # Astroid can fail on this file, specifically raising an exception at this line of code:
+            #     lambda node: node.name == "NamedTuple" and node.parent.name == "typing"
+            # with the error:
+            #     AttributeError: 'If' object has no attribute 'name'
+            # See https://github.com/gristlabs/asttokens/runs/7602147792
+            # I think the code that causes the problem is:
+            #     if sys.version_info >= (3, 11):
+            #         NamedTuple = typing.NamedTuple
+            or filename.endswith("typing_extensions.py")
+        ):
           print('Skipping', filename)
           continue
 
@@ -618,6 +657,80 @@ j  # not a complex number, just a name
     def test_dict_merge(self):
       self.create_mark_checker("{**{}}")
 
+    def test_async_def(self):
+      self.create_mark_checker("""
+async def foo():
+  pass
+
+@decorator
+async def foo():
+  pass
+""")
+
+    def test_async_for_and_with(self):
+      # Can't verify all nodes because in < 3.7
+      # async for/with outside of a function is invalid syntax
+      m = self.create_mark_checker("""
+async def foo():
+  async for x in y: pass
+  async with x as y: pass
+  """, verify=False)
+      assert m.view_nodes_at(3, 2) == {"AsyncFor:async for x in y: pass"}
+      assert m.view_nodes_at(4, 2) == {"AsyncWith:async with x as y: pass"}
+
+    def test_await(self):
+      # Can't verify all nodes because in astroid
+      # await outside of an async function is invalid syntax
+      m = self.create_mark_checker("""
+async def foo():
+  await bar
+  """, verify=False)
+      assert m.view_nodes_at(3, 2) == {"Await:await bar", "Expr:await bar"}
+
+  if sys.version_info >= (3, 8):
+    def test_assignment_expressions(self):
+      # From https://www.python.org/dev/peps/pep-0572/
+      self.create_mark_checker("""
+# Handle a matched regex
+if (match := pattern.search(data)) is not None:
+    # Do something with match
+    pass
+
+# A loop that can't be trivially rewritten using 2-arg iter()
+while chunk := file.read(8192):
+   process(chunk)
+
+# Reuse a value that's expensive to compute
+[y := f(x), y**2, y**3]
+
+# Share a subexpression between a comprehension filter clause and its output
+filtered_data = [y for x in data if (y := f(x)) is not None]
+
+y0 = (y1 := f(x))  # Valid, though discouraged
+
+foo(x=(y := f(x)))  # Valid, though probably confusing
+
+def foo(answer=(p := 42)):  # Valid, though not great style
+    ...
+
+def foo(answer: (p := 42) = 5):  # Valid, but probably never useful
+    ...
+
+lambda: (x := 1) # Valid, but unlikely to be useful
+
+(x := lambda: 1) # Valid
+
+lambda line: (m := re.match(pattern, line)) and m.group(1) # Valid
+
+if any((comment := line).startswith('#') for line in lines):
+    print("First comment:", comment)
+
+if all((nonblank := line).strip() == '' for line in lines):
+    print("All lines are blank")
+
+partial_sums = [total := total + v for v in values]
+""")
+
   def parse_snippet(self, text, node):
     """
     Returns the parsed AST tree for the given text, handling issues with indentation and newlines
@@ -630,9 +743,15 @@ j  # not a complex number, just a name
     # For expressions and statements, we add a dummy statement '_' before it because if it's just a
     # string contained in an astroid.Const or astroid.Expr it will end up in the doc attribute and be
     # a pain to extract for comparison
+    # For starred expressions, e.g. `*args`, we wrap it in a function call to make it parsable.
+    # For slices, e.g. `x:`, we wrap it in an indexing expression to make it parsable.
     indented = re.match(r'^[ \t]+\S', text)
     if indented:
       return self.module.parse('def dummy():\n' + text).body[0].body[0]
+    if util.is_starred(node):
+      return self.module.parse('f(' + text + ')').body[0].value.args[0]
+    if util.is_slice(node):
+      return self.module.parse('a[' + text + ']').body[0].value.slice
     if util.is_expr(node):
       return self.module.parse('_\n(' + text + ')').body[1].value
     if util.is_module(node):
